"""
This module contains the concrete response handlers for both DRF and Django Ninja responses.
"""

import json
from typing import TYPE_CHECKING, Optional, Union

if TYPE_CHECKING:
    from django.http.response import HttpResponse
    from rest_framework.response import Response


class ResponseHandler:
    """
    This class is used to handle the response and request data
    from both DRF and Django HTTP (Django Ninja) responses.
    """

    def __init__(self, response: Union["Response", "HttpResponse"]) -> None:
        self._response = response

    @property
    def response(self) -> Union["Response", "HttpResponse"]:
        return self._response

    @property
    def data(self) -> Optional[dict]:
        ...

    @property
    def request_data(self) -> Optional[dict]:
        ...


class DRFResponseHandler(ResponseHandler):
    """
    Handles the response and request data from DRF responses.
    """

    def __init__(self, response: "Response") -> None:
        super().__init__(response)

    @property
    def data(self) -> Optional[dict]:
        return self.response.json() if self.response.data is not None else None  # type: ignore[attr-defined]

    @property
    def request_data(self) -> Optional[dict]:
        return self.response.renderer_context["request"].data  # type: ignore[attr-defined]


class DjangoNinjaResponseHandler(ResponseHandler):
    """
    Handles the response and request data from Django Ninja responses.
    """

    def __init__(self, response: "HttpResponse") -> None:
        super().__init__(response)

    @property
    def data(self) -> Optional[dict]:
        return self.response.json() if self.response.content else None  # type: ignore[attr-defined]

    @property
    def request_data(self) -> Optional[dict]:
        response_body = self.response.wsgi_request.body  # type: ignore[attr-defined]
        return json.loads(response_body) if response_body else None



"""Subclass of ``APIClient`` using ``SchemaTester`` to validate responses."""

from __future__ import annotations

from typing import TYPE_CHECKING

from rest_framework.test import APIClient

from .schema_tester import SchemaTester

if TYPE_CHECKING:
    from rest_framework.response import Response


class OpenAPIClient(APIClient):
    """``APIClient`` validating responses against OpenAPI schema."""

    def __init__(
        self,
        *args,
        schema_tester: SchemaTester | None = None,
        **kwargs,
    ) -> None:
        """Initialize ``OpenAPIClient`` instance."""
        super().__init__(*args, **kwargs)
        self.schema_tester = schema_tester or self._schema_tester_factory()

    def request(self, **kwargs) -> Response:  # type: ignore[override]
        """Validate fetched response against given OpenAPI schema."""
        response = super().request(**kwargs)
        if self._is_successful_response(response):
            self.schema_tester.validate_request(response)
        self.schema_tester.validate_response(response)
        return response

    @staticmethod
    def _is_successful_response(response: Response) -> bool:
        return response.status_code < 400

    @staticmethod
    def _schema_tester_factory() -> SchemaTester:
        """Factory of default ``SchemaTester`` instances."""
        return SchemaTester()
